package com.kob.backend.consumer;

import com.alibaba.fastjson.JSONObject;
import com.kob.backend.consumer.utils.Game;
import com.kob.backend.consumer.utils.JwtAuthentication;
import com.kob.backend.pojo.Bot;
import com.kob.backend.pojo.User;
import com.kob.backend.mapper.BotMapper;
import com.kob.backend.mapper.RecordMapper;
import com.kob.backend.mapper.UserMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.client.RestTemplate;

import javax.websocket.*;
import javax.websocket.server.PathParam;
import javax.websocket.server.ServerEndpoint;
import java.io.IOException;
import java.util.concurrent.ConcurrentHashMap;

/**
 * @author zfp
 */
@Component
@ServerEndpoint("/websocket/{token}")  // 注意不要以'/'结尾
public class WebSocketServer {

    // 保存所有用户对应的请求链接
    // 线程安全的一个HashMap
    public final static ConcurrentHashMap<Integer,WebSocketServer> users = new ConcurrentHashMap<>();
    // 发起请求链接对应的用户
    private User user;
    private Session session = null;
    public Game game = null;

    // 注入userMapper(和之前的有点不一样)
    public static UserMapper userMapper;
    // 注入RecordMapper
    public static RecordMapper recordMapper;
    // 注入BotMapper
    private static BotMapper botMapper;

    // 注入RestTemplate
    public static RestTemplate restTemplate; // 可以作用于两个springboot进程间的通信

    private static final String addPlayerUrl = "http://127.0.0.1:4001/player/add/";
    private static final String removePlayerUrl = "http://127.0.0.1:4001/player/remove/";


    @Autowired
    public void setUserMapper(UserMapper userMapper) {
        WebSocketServer.userMapper = userMapper;
    }

    @Autowired
    public void setRecordMapper(RecordMapper recordMapper) {
        WebSocketServer.recordMapper = recordMapper;
    }

    @Autowired
    private void setRestTemplate(RestTemplate restTemplate) {
        WebSocketServer.restTemplate = restTemplate;
    }

    @Autowired
    private void setBotMapper(BotMapper botMapper) {
        WebSocketServer.botMapper = botMapper;
    }
    @OnOpen
    public void onOpen(Session session, @PathParam("token") String token) throws IOException {
        // 建立连接
        this.session = session;

        // 根据token查询用户id(JWT验证)
        Integer userId = JwtAuthentication.getUserId(token);
        // 根据userId查找对应的用户
        this.user = userMapper.selectById(userId);
        // 连接成功
        if (this.user != null) {
            users.put(userId,this);
            System.out.println("Connected Success!");
        }else {
            this.session.close();
        }

        System.out.println(users);
    }

    @OnClose
    public void onClose() {
        // 关闭链接
        System.out.println("Disconnected!");
        if (this.user != null) {
            users.remove(this.user.getId());
        }
    }
    public static void startGame(Integer aId, Integer a_bot_id,Integer bId,Integer b_bot_id) {
        User a = userMapper.selectById(aId),b = userMapper.selectById(bId);
        Bot bot_a = botMapper.selectById(a_bot_id),bot_b = botMapper.selectById(b_bot_id);

        Game game = new Game(13,14,20,a.getId(),b.getId(),bot_a,bot_b);
        game.createMap();
        if (users.get(a.getId()) != null)
            users.get(a.getId()).game = game;
        if (users.get(b.getId()) != null)
            users.get(b.getId()).game = game;

        game.start(); // 重新开一个新线程



        // 两名玩家的id，位置以及地图保存在respGame中
        JSONObject respGame = new JSONObject();
        respGame.put("a_id",game.getPlayerA().getId());
        respGame.put("a_sx",game.getPlayerA().getSx());
        respGame.put("a_sy",game.getPlayerA().getSy());
        respGame.put("b_id",game.getPlayerB().getId());
        respGame.put("b_sx",game.getPlayerB().getSx());
        respGame.put("b_sy",game.getPlayerB().getSy());
        respGame.put("map",game.getG());


        JSONObject respA = new JSONObject();
        respA.put("event","start-matching");
        respA.put("opponent_username",b.getUsername());
        respA.put("opponent_photo",b.getPhoto());
        respA.put("game",respGame);
        if (users.get(a.getId()) != null)
            users.get(a.getId()).sendMessage(respA.toJSONString());

        JSONObject respB = new JSONObject();
        respB.put("event","start-matching");
        respB.put("opponent_username",a.getUsername());
        respB.put("opponent_photo",a.getPhoto());
        respB.put("game",respGame);
        if (users.get(b.getId()) != null)
            users.get(b.getId()).sendMessage(respB.toJSONString());
    }


    private void startMatching(Integer botId) {
        System.out.println("start-----matching----");
        MultiValueMap<String,String> data = new LinkedMultiValueMap<>();

        // 请求参数
        data.add("user_id",this.user.getId().toString());
        data.add("rating",this.user.getRating().toString());
        data.add("bot_id",botId.toString());

        // 向后端发送请求，添加一个用户
        restTemplate.postForObject(addPlayerUrl,data,String.class);

    }

    private void stopMatching() {
        System.out.println("stop-----matching----");
        // 向后端发送请求，删除一个用户
        MultiValueMap<String,String> data = new LinkedMultiValueMap<>();
        data.add("user_id",this.user.getId().toString());
        restTemplate.postForObject(removePlayerUrl,data,String.class);
    }

    private void move(int direction) {
        if (game.getPlayerA().getId().equals(user.getId())) {
            if (game.getPlayerA().getBotId().equals(-1)) { // 如果是用户出战
                game.setNextStepA(direction);
            }
        }else if (game.getPlayerB().getId().equals(user.getId())) {
            if (game.getPlayerB().getBotId().equals(-1)) { // 如果是用户出战
                game.setNextStepB(direction);
            }
        }
    }

    @OnMessage
    public void onMessage(String message, Session session) {
        // 从Client接收消息(读取前端发来的信息)
        System.out.println("Received the message!");
        JSONObject data = JSONObject.parseObject(message);
        String event = data.getString("event");

        if ("start-matching".equals(event)) {
            startMatching(data.getInteger("bot_id"));
        }
        else if ("stop-matching".equals(event)){
            stopMatching();
        }else if ("move".equals(event)) {
            move(data.getInteger("direction"));
        }
    }

    @OnError
    public void onError(Session session, Throwable error) {
        error.printStackTrace();
    }

    public void sendMessage(String msg) {
        // 给前端发送信息
        synchronized (this.session) {
            try {
                this.session.getBasicRemote().sendText(msg);
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }
}