package com.kob.backend.service.impl.user.bot;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.kob.backend.pojo.Bot;
import com.kob.backend.pojo.User;
import com.kob.backend.service.impl.utils.UserDetailsImpl;
import com.kob.backend.service.user.bot.AddService;
import com.kob.backend.mapper.BotMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * @author zfp
 */
@Service
public class AddServiceImpl implements AddService {
    @Autowired
    private BotMapper botMapper;

    @Override
    public Map<String, String> add(Map<String, String> data) {
        // 获取当前登录的是哪个用户(三条语句)
        UsernamePasswordAuthenticationToken authenticationToken = (UsernamePasswordAuthenticationToken)
                SecurityContextHolder.getContext().getAuthentication();
        UserDetailsImpl loginUser = (UserDetailsImpl) authenticationToken.getPrincipal();
        User user = loginUser.getUser();

        String title = data.get("title");
        String description = data.get("description");
        String content = data.get("content");

        Map<String, String> map = new HashMap<>();
        if (title == null || title.length() == 0) {
            map.put("error_msg", "标题不能为空!");
            return map;
        }

        int TITLE_MAX_LEN = 100;
        if (title.length() > TITLE_MAX_LEN) {
            map.put("error_msg", "标题长度不能大于100!");
            return map;
        }

        if (description == null || description.length() == 0) {
            description = "这个用户很懒，什么也没留下~";
        }

        int DESCRIPTION_MAX_LEN = 300;
        if (description.length() > DESCRIPTION_MAX_LEN) {
            map.put("error_msg", "bot的描述不能大于300!");
            return map;
        }


        if (content == null || content.length() == 0) {
            map.put("error_msg", "代码不能为空!");
            return map;
        }

        int CONTENT_MAX_LEN = 10000;
        if (content.length() > CONTENT_MAX_LEN) {
            map.put("error_msg", "代码长度不能大于10000!");
            return map;
        }

        QueryWrapper<Bot> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("user_id",user.getId());

        if (botMapper.selectCount(queryWrapper) >= 10) {
            map.put("error_msg","每名用户最多只能创建10个bot!");
            return map;
        }

        Date now = new Date();
        Bot bot = new Bot(null, user.getId(), title, description, content, now, now);
        botMapper.insert(bot);

        map.put("error_msg", "success");
        return map;
    }
}
