package com.kob.backend.service.impl.user.bot;

import com.kob.backend.pojo.Bot;
import com.kob.backend.pojo.User;
import com.kob.backend.service.impl.utils.UserDetailsImpl;
import com.kob.backend.service.user.bot.UpdateService;
import com.kob.backend.mapper.BotMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * @author zfp
 */
@Service
public class UpdateServiceImpl implements UpdateService {
    @Autowired
    private BotMapper botMapper;


    @Override
    public Map<String, String> update(Map<String, String> data) {
        // 获得登录的用户信息
        UsernamePasswordAuthenticationToken authenticationToken = (UsernamePasswordAuthenticationToken)
                SecurityContextHolder.getContext().getAuthentication();
        UserDetailsImpl loginUser = (UserDetailsImpl) authenticationToken.getPrincipal();
        User user = loginUser.getUser();
        // 需要知道更新哪个bot，所以要传入bot_id
        int bot_id = Integer.parseInt(data.get("bot_id"));

        Map<String,String> map = new HashMap<>();

        String title = data.get("title");
        String description = data.get("description");
        String content = data.get("content");

        if (title == null || title.length() == 0) {
            map.put("error_msg", "标题不能为空!");
            return map;
        }

        int TITLE_MAX_LEN = 100;
        if (title.length() > TITLE_MAX_LEN) {
            map.put("error_msg", "标题长度不能大于100!");
            return map;
        }

        if (description == null || description.length() == 0) {
            description = "这个用户很懒，什么也没留下~";
        }

        int DESCRIPTION_MAX_LEN = 300;
        if (description.length() > DESCRIPTION_MAX_LEN) {
            map.put("error_msg", "bot的描述不能大于300!");
            return map;
        }


        if (content == null || content.length() == 0) {
            map.put("error_msg", "代码不能为空!");
            return map;
        }

        int CONTENT_MAX_LEN = 10000;
        if (content.length() > CONTENT_MAX_LEN) {
            map.put("error_msg", "代码长度不能大于10000!");
            return map;
        }

        // 根据用户传入的bot_id查询数据库中是否存在该bot
        Bot bot = botMapper.selectById(bot_id);

        if (bot == null) {
            map.put("error_msg","bot不存在或已被删除");
            return map;
        }

        // 如果该bot不属于发出更新请求的用户，提示无权限
        if (!bot.getUserId().equals(user.getId())){
            map.put("error_msg","您没有权限更新该bot的信息");
            return map;
        }

        // 如果可以修改该bot
        Bot newBot = new Bot(bot.getId(), user.getId(), title,description,content,bot.getCreateTime(),new Date());
        botMapper.updateById(newBot);
        map.put("error_msg","success");
        return map;
    }
}
