package com.kob.matchingsystem.service.impl.utils;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.client.RestTemplate;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.locks.ReentrantLock;
@Component
public class MatchingPool extends Thread{

    public static List<Player> players = new ArrayList<>();
    // 加锁
    public ReentrantLock lock = new ReentrantLock();

    private static RestTemplate restTemplate;

    private static final String startGameUrl= "http://127.0.0.1:4000/pk/start/game/";

    @Autowired
    public void setRestTemplate(RestTemplate restTemplate) {
        MatchingPool.restTemplate = restTemplate;
    }

    public void addPlayer(Integer userId,Integer rating,Integer botId) {
        lock.lock();
        try {
            players.add(new Player(userId,rating,botId,0));
        }finally {
            lock.unlock();
        }
    }

    public void removePlayer(Integer userId){
        lock.lock();
        try {
            List<Player> newPlayers = new ArrayList<>();
            for (Player player: players) {
                if (!player.getUserId().equals(userId)) {
                    newPlayers.add(player);
                }
            }
            players = newPlayers;
        }finally {
            lock.unlock();
        }
    }

    private void increaseWaitingTime() {
        // 所有等待的用户等待时间加1
        for (Player p : players) {
            p.setWaitingTime(p.getWaitingTime() + 1);
        }
    }

    // 判断两名玩家是否匹配
    private boolean checkMatched(Player a,Player b) {
        int ratingDelta = Math.abs(a.getRating() - b.getRating()); // 分差
        int waitingTime = Math.min(a.getWaitingTime(),b.getWaitingTime()); // 同时满足两名用户的要求
        return ratingDelta < waitingTime * 10;
    }

    // 尝试匹配玩家
    private void matchPlayer() {
        boolean[] used = new boolean[players.size()]; // 记录用户是否已经被匹配

        for (int i = 0; i < players.size(); i++) {
            if (used[i]) continue;
            for (int j = i + 1; j < players.size(); j++) {
                if (used[j]) continue;
                Player a = players.get(i), b = players.get(j);
                if (checkMatched(a,b)) {
                    used[i] = used[j] = true;
                    sendResult(a,b);
                    break;
                }
            }
        }
        // 删除已经匹配成功的用户
        List<Player> newPlayers = new ArrayList<>();
        for (int i = 0; i < players.size(); i++) {
            if (!used[i]) newPlayers.add(players.get(i));
        }

        players = newPlayers;
    }

    // 返回两名玩家的匹配结果
    private void sendResult(Player a,Player b) {
        System.out.println("send result " + a + " " + b);
        MultiValueMap<String,String> data = new LinkedMultiValueMap<>();
        data.add("a_id",a.getUserId().toString());
        data.add("a_bot_id",a.getBotId().toString());
        data.add("b_id",b.getUserId().toString());
        data.add("b_bot_id",b.getBotId().toString());
        // 向WebServer返回匹配结果
        restTemplate.postForObject(startGameUrl,data,String.class);

    }

    @Override
    public void run() {
        while (true) {
            try {
                Thread.sleep(1000);
                lock.lock();
                try{
                    increaseWaitingTime();
                    matchPlayer();
                }finally {
                    lock.unlock();
                }
            } catch (InterruptedException e) {
                e.printStackTrace();
                break;
            }
        }
    }
}
